"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const crypto_1 = __importDefault(require("crypto"));
const fs_1 = __importDefault(require("fs"));
const nextPowerOfTwo_1 = require("../util/nextPowerOfTwo");
const optionalMap = (val, mapper) => {
    return val == undefined ? undefined : mapper(val);
};
exports.parseAWSS3GlacierOptions = (args) => {
    const region = optionalMap(args.region, v => v.toString());
    const accessId = optionalMap(args.access, v => v.toString());
    const secretKey = optionalMap(args.secret, v => v.toString());
    const vaultName = args.vault.toString();
    return {
        region, accessId, secretKey, vaultName,
    };
};
const buildTreeHash = (hashes) => {
    while (hashes.length > 1) {
        const newHashes = [];
        for (let i = 0; i < hashes.length - 1; i += 2) {
            newHashes.push(crypto_1.default.createHash("sha256")
                .update(Buffer.concat(hashes.slice(i, i + 2)))
                .digest());
        }
        if (hashes.length % 2 === 1) {
            newHashes.push(hashes[hashes.length - 1]);
        }
        hashes = newHashes;
    }
    return hashes[0];
};
const MiB = 1024 * 1024;
// This is difficult, as it needs to build the tree hash while reading the part,
// not collect hashes for the entire part first (which might result in heavy
// memory usage for large part sizes).
const calculateTreeAndLinearHashOfPart = (p) => { var p_1, p_1_1; return __awaiter(this, void 0, void 0, function* () {
    var e_1, _a;
    const tree = [];
    const linear = crypto_1.default.createHash("sha256");
    let leftover = Buffer.alloc(0);
    const handleChunk = (chunk) => {
        // WARNING: Don't assign to or use outer scope $leftover in this function,
        // as $chunk argument might be $leftover.
        // Build linear hash.
        linear.update(chunk);
        // Calculate tree hash leaf.
        tree.push({
            level: 1,
            hash: crypto_1.default.createHash("sha256").update(chunk).digest(),
        });
        // Merge hashes during tree building.
        while (tree.length > 1 && tree[tree.length - 1].level === tree[tree.length - 2].level) {
            const right = tree.pop();
            const left = tree.pop();
            tree.push({
                level: right.level + 1,
                hash: crypto_1.default.createHash("sha256")
                    .update(Buffer.concat([left.hash, right.hash]))
                    .digest(),
            });
        }
    };
    try {
        for (p_1 = __asyncValues(p); p_1_1 = yield p_1.next(), !p_1_1.done;) {
            let chunk = p_1_1.value;
            // Even though the highWaterMark should be set to 1 MiB,
            // it's allowed that the ReadStream only reads < 1 MiB for a chunk.
            if (chunk.length > MiB) {
                // This should never happen.
                throw new Error(`ReadStream chunk is larger than 1 MiB`);
            }
            // Combine read chunk with previous leftover.
            chunk = Buffer.concat([leftover, chunk]);
            if (chunk.length < MiB) {
                // Combined chunk is smaller than 1 MiB, not ready to process yet.
                leftover = chunk;
                continue;
            }
            // Process only first 1 MiB of combined chunk and leave remaining for later.
            leftover = chunk.slice(MiB);
            chunk = chunk.slice(0, MiB);
            handleChunk(chunk);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (p_1_1 && !p_1_1.done && (_a = p_1.return)) yield _a.call(p_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    if (leftover.length) {
        handleChunk(leftover);
    }
    while (tree.length > 1) {
        const right = tree.pop();
        const left = tree.pop();
        tree.push({
            level: -1,
            hash: crypto_1.default.createHash("sha256")
                .update(Buffer.concat([left.hash, right.hash]))
                .digest(),
        });
    }
    return {
        tree: tree[0].hash,
        linear: linear.digest(),
    };
}); };
// Patch internal method addTreeHashHeaders on AWS.Glacier class to support
// a body of type `fs.ReadStream & {psf: PartStreamFactory}`. By default, the
// SDK tries to convert the ReadStream into a Buffer, which fails as Buffer.from
// doesn't support an argument of type fs.ReadStream. In addition, we don't want
// to read the entire part (which could be up to 4 GiB) into memory, so we pass
// a ReadStream as the body value (which the SDK does support uploading from) with
// a `psf` property that, when called, creates a new ReadStream of the same part,
// as the initial ReadStream will be consumed when building the hashes.
aws_sdk_1.default.Glacier.prototype.addTreeHashHeaders = (request, callNextListener) => __awaiter(this, void 0, void 0, function* () {
    const body = request.params.body;
    if (body instanceof fs_1.default.ReadStream && typeof body.psf == "function") {
        const { linear, tree } = yield calculateTreeAndLinearHashOfPart(request.params.body.psf(MiB));
        // The property "X-Amz-Content-Sha256" needs to be exactly in that letter casing,
        // otherwise the SDK will think it doesn't exist and try to hash the ReadStream (which fails).
        request.httpRequest.headers["X-Amz-Content-Sha256"] = linear.toString("hex");
        if (request.httpRequest.headers["x-amz-sha256-tree-hash"]) {
            throw new Error(`Tree hash header already exists on AWS Glacier request`);
        }
        request.httpRequest.headers["x-amz-sha256-tree-hash"] = tree.toString("hex");
        // Length property is needed as by default the SDK gets the length of a fs.ReadStream by the size of the
        // file at `stream._path`.
        request.params.body = Object.assign(request.params.body.psf(), { length: request.params.body.length });
    }
    else if (body != undefined) {
        const { linearHash, treeHash } = request.service.computeChecksums(request.params.body);
        request.httpRequest.headers["X-Amz-Content-Sha256"] = linearHash;
        if (!request.httpRequest.headers["x-amz-sha256-tree-hash"]) {
            request.httpRequest.headers["x-amz-sha256-tree-hash"] = treeHash;
        }
    }
    callNextListener();
});
// The AWS SDK uses this to determine that the function is asynchronous and so will provide
// a callback and wait for it to be called with any error before continuing.
// This is necessary as our calculateTreeAndLinearHashOfPart function is asynchronous.
aws_sdk_1.default.Glacier.prototype.addTreeHashHeaders._isAsync = true;
exports.AWSS3Glacier = {
    // See https://docs.aws.amazon.com/amazonglacier/latest/dev/uploading-archive-mpu.html for limits.
    minParts: 1,
    maxParts: 10000,
    minPartSize: 1024 * 1024,
    maxPartSize: 1024 * 1024 * 1024 * 4,
    fromOptions(options) {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                service: new aws_sdk_1.default.Glacier({
                    accessKeyId: options.accessId || undefined,
                    secretAccessKey: options.secretKey || undefined,
                    region: options.region || undefined,
                }),
                vaultName: options.vaultName,
            };
        });
    },
    completeUpload(s, uploadId, fileSize, partHashes) {
        return __awaiter(this, void 0, void 0, function* () {
            yield s.service.completeMultipartUpload({
                accountId: "-",
                archiveSize: `${fileSize}`,
                checksum: buildTreeHash(partHashes).toString("hex"),
                uploadId: uploadId,
                vaultName: s.vaultName,
            }).promise();
        });
    },
    idealPartSize(_, fileSize) {
        return __awaiter(this, void 0, void 0, function* () {
            return nextPowerOfTwo_1.nextPowerOfTwo(fileSize / this.maxParts);
        });
    },
    initiateNewUpload(s, fileName, partSize) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield s.service.initiateMultipartUpload({
                accountId: "-",
                vaultName: s.vaultName,
                partSize: `${partSize}`,
                archiveDescription: fileName,
            }).promise()).uploadId;
        });
    },
    uploadPart(s, uploadId, psf, { start, end }) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield s.service.uploadMultipartPart({
                accountId: "-",
                // See comment above the AWS.Glacier.prototype.addTreeHashHeaders patch above to see why.
                body: Object.assign(psf(), { psf, length: end - start + 1 }),
                range: `bytes ${start}-${end}/*`,
                uploadId: uploadId,
                vaultName: s.vaultName,
            }).promise();
            const serverSentTreeHash = Buffer.from(res.checksum, "hex");
            return {
                hash: serverSentTreeHash,
            };
        });
    }
};
//# sourceMappingURL=AWSS3Glacier.js.map