"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const request_1 = __importDefault(require("request"));
class HTTPBadStatusError extends Error {
    constructor(status, body, url) {
        super(`${url} returned bad HTTP status of ${status}: ${body}`);
        this.statusCode = status;
    }
}
exports.HTTPBadStatusError = HTTPBadStatusError;
const isPlainObject = (o) => {
    if (o == null) {
        return false;
    }
    const proto = Object.getPrototypeOf(o);
    return proto == Object.prototype || proto == null;
};
exports.http = ({ method, url, headers, body }) => {
    return new Promise((resolve, reject) => {
        if (isPlainObject(body)) {
            body = JSON.stringify(body);
        }
        request_1.default({
            url,
            method,
            headers,
            body,
            timeout: 120000,
        }, (err, res) => {
            if (err) {
                reject(err);
            }
            else if (res.statusCode < 200 || res.statusCode > 299) {
                reject(new HTTPBadStatusError(res.statusCode, res.body, url));
            }
            else {
                const parsed = JSON.parse(res.body);
                resolve(parsed);
            }
        });
    });
};
//# sourceMappingURL=http.js.map